#!/usr/local/bin/perl5 -w

# This program will build a particular collection

BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    die "GSDLOS not set\n" unless defined $ENV{'GSDLOS'};
    unshift (@INC, "$ENV{'GSDLHOME'}/lib");
    unshift (@INC, "$ENV{'GSDLHOME'}/lib/plugins");
}

use parsargv;
use util;

&main();

sub print_usage {
    print STDERR "\n  usage: $0 [options] collection-name\n\n";
    print STDERR "  options:\n";
    print STDERR "   -verbosity number     0=none, 3=lots\n";
    print STDERR "   -archivedir directory Where the archives live\n";
    print STDERR "   -cachedir directory   Where to cache the archives\n";
    print STDERR "   -builddir directory   Where to put the built indexes\n\n";
}


sub main
{
    if (!parsargv::parse(\@ARGV, 
			 'verbosity/\d+/2', \$verbosity,
			 'archivedir/.*/', \$archivedir,
			 'cachedir/.*/', \$cachedir,
			 'builddir/.*/', \$builddir)) {
	&print_usage();
	die "\n";
    }

    # get and check the collection
    ($collection) = @ARGV;
    if (!defined ($collection)) {
	&print_usage();
	die "\n";
    }
    if (!-e "$ENV{'GSDLHOME'}/collect/$collection") {
	die "Invalid collection name ($collection).\n";
    }

    # fill in the default archives and building directories if none
    # were supplied, turn all \ into / and remove trailing /
    $archivedir = "$ENV{'GSDLHOME'}/collect/$collection/archives" if $archivedir eq "";
    $archivedir =~ s/[\\\/]+/\//g;
    $archivedir =~ s/\/$//;
    $builddir = "$ENV{'GSDLHOME'}/collect/$collection/building" if $builddir eq "";
    $builddir =~ s/[\\\/]+/\//g;
    $builddir =~ s/\/$//;

    # update the archive cache if needed
    if ($cachedir) {
	print STDERR "Updating archive cache\n" if ($verbosity >= 1);

	$cachedir =~ s/[\\\/]+$//;
	$cachedir .= "/collect/$collection" unless 
	    $cachedir =~ /collect\/$collection/;

	$realarchivedir = "$cachedir/archives";
	$realbuilddir = "$cachedir/building";
	&util::mk_all_dir ($realarchivedir);
	&util::mk_all_dir ($realbuilddir);
	&util::cachedir ($archivedir, $realarchivedir, $verbosity);

    } else {
	$realarchivedir = $archivedir;
	$realbuilddir = $builddir;
    }

    # build it in realbuilddir
    &util::mk_all_dir ($realbuilddir);


    # if a builder class has been created for this collection, use it
    # otherwise, use the mg builder
    if (-e "$ENV{'GSDLHOME'}/collect/$collection/lib/${collection}builder.pm") {
	$builderdir = "$ENV{'GSDLHOME'}/collect/$collection/lib";
	$buildertype = "${collection}builder";
    } else {
	$builderdir = "$ENV{'GSDLHOME'}/lib";
	$buildertype = "mgbuilder";
    }
	
    require "$builderdir/$buildertype.pm";

    eval("\$builder = new $buildertype(\$collection, " .
	 "\$realarchivedir, \$realbuilddir, \$verbosity)");
    die "$@" if $@;

    $builder->init();
    $builder->compress_text();
    $builder->build_indexes();
    $builder->make_infodatabase();
    $builder->make_auxiliary_files();
    $builder->deinit();


    if ($realbuilddir ne $builddir) {
	print STDERR "Copying back the cached build\n" if ($verbosity >= 1);
	&util::rm_r ($builddir);
	&util::cp_r ($realbuilddir, $builddir);
    }
}


